from .get_static_elements import get_static_elements
from .main import get_source_text_elements
from .set_static_elements import set_static_elements
from .text_elements import TextElement, TextElements

__all__ = [
    "TextElement",
    "TextElements",
    "get_source_text_elements",
    "get_static_elements",
    "set_static_elements",
]
